@@ -509,17 +509,24 @@ class Testbed:
             except adtlog.AptError:
                 self.bomb("failed testbed apt update")
 
-        if self.add_apt_releases or self.add_apt_pockets:
-            # Assume the testbed release to be the one that provides the candidate
-            # base-files version. This requires the package index to be up-to-date.
-            _, _, self.release = self.get_candidate("base-files")
-            if self.release:
-                adtlog.info(f"testbed release detected to be: {self.release}")
-            else:
-                self.bomb(
-                    "could not determine the testbed release (no APT source for base-files?)",
-                    adtlog.AutopkgtestError,
-                )
+        # Assume the testbed release to be the one that provides the candidate
+        # base-files version. This requires the package index to be up-to-date.
+        _, _, self.release = self.get_candidate("base-files")
+        if self.release:
+            adtlog.info(f"testbed release detected to be: {self.release}")
+        else:
+            adtlog.info(
+                "could not determine the testbed release (no APT source for base-files?)"
+            )
+        if not self.release and (
+            self.add_apt_releases
+            or self.add_apt_pockets
+            or any(self.PPA_PATTERN.match(s) for s in self.add_apt_sources)
+        ):
+            self.bomb(
+                "testbed release required to be known for autopkgtest to run",
+                adtlog.AutopkgtestError,
+            )
 
         pin_packages = self.pin_packages.copy()
         add_apt_releases = self.add_apt_releases.copy()
@@ -1763,7 +1770,7 @@ class Testbed:
         if self.output_dir:
             ap = TestbedPath(
                 self,
-                os.path.join(self.output_dir, "artifacts"),
+                os.path.join(self.output_dir, "artifacts", test.name),
                 test_artifacts,
                 is_dir=True,
             )
@@ -1771,6 +1778,8 @@ class Testbed:
             # don't keep an empty artifacts dir around
             if not os.listdir(ap.host):
                 os.rmdir(ap.host)
+            if not os.listdir(os.path.dirname(ap.host)):
+                os.rmdir(os.path.dirname(ap.host))
 
         if shell or (shell_on_failure and not test.result):
             self.run_shell(
