@@ -106,7 +106,8 @@ wait_booted() {
         exit 1
     fi
 
-    if ! timeout 300 "$COMMAND" exec "${REMOTE:+$REMOTE:}$CONTAINER" -- sh -s < "$await_boot"; then
+    # shellcheck disable=SC2002
+    if ! cat "$await_boot" | timeout 300 "$COMMAND" exec "${REMOTE:+$REMOTE:}$CONTAINER" -- sh -s; then
         echo "Error or timeout waiting for instance to boot" >&2
         exit 1
     fi
@@ -144,12 +145,13 @@ setup() {
                   /usr/share/autopkgtest/setup-commands/setup-testbed; do
         if [ -r "$script" ]; then
             echo "Running setup script $script..."
-            "$COMMAND" exec "${REMOTE:+$REMOTE:}$CONTAINER" -- env \
+            # shellcheck disable=SC2002
+            cat "$script" | "$COMMAND" exec "${REMOTE:+$REMOTE:}$CONTAINER" -- env \
                 AUTOPKGTEST_KEEP_APT_SOURCES="${AUTOPKGTEST_KEEP_APT_SOURCES:-}" \
                 AUTOPKGTEST_APT_SOURCES="${AUTOPKGTEST_APT_SOURCES:-}" \
                 MIRROR="${MIRROR:-}" \
                 RELEASE="${RELEASE}" \
-                sh < "$script"
+                sh -s
             break
         fi
     done
